/*
Thirteen Spikes Unlimited version 0.8.0
Copyright (c) 2008-2009 Blazing Games Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package
{
	import flash.display.MovieClip;
	import flash.display.Bitmap;
	import flash.events.Event;
	import flash.text.TextField;
	import flash.utils.getTimer;

	import com.blazinggames.tsu.*;
	import com.blazinggames.tsu.skins.Skin;
	import com.blazinggames.tsu.skins.skyskin.SkySkin;

	[SWF(width="640", height="480", backgroundColor="#000000")]
	
	public class ThirteenSpikesUnlimited extends MovieClip
	{
// ***** CONSTANTS ***********************************************************

		public static const MODE_NONE:int = 0;
		public static const MODE_TITLE:int = 1;
		public static const MODE_GAME:int = 2;
		public static const MODE_CAMPAIGN:int = 3;
		public static const MODE_SCORE:int = 4;
		public static const MODE_ABOUT:int = 5;
		public static const MODE_EDIT:int = 6;

// ***** VARIABLES ***********************************************************
		
		private var _lastUpdate:int;
		private var _frameRateDisplay:TextField;
		private var _mode:int;
		private var _game:Level;
		public var _campaign:Campaign;
		private var _title:Title;
		private var _score:Score;
		private var _about:About;
		private var _edit:EditMenu;
		private var _skin:Skin;

		private var _editing:Boolean;
		
// ***** CONSTRUCTOR *********************************************************

		public function ThirteenSpikesUnlimited()
		{
			_editing = false;
			_skin = new SkySkin();
			_skin.addEventListener(Skin.SKIN_LOADED, handleSkinLoaded);
			_skin.loadSkin();
//			addChild(_skin.downloadedImage);
			
			_lastUpdate = getTimer();
			trace("Starting 13 spikes unlimited");
			_mode = MODE_NONE;
			_game = new Level(this);
			_campaign = new Campaign(this);
//			_game.generateLevel(2);
			_title = new Title(this);
			_score = new Score(this);
			_about = new About(this);
			_edit = new EditMenu(this);
			changeMode(MODE_TITLE);
			stop();
			stage.addEventListener(Event.ENTER_FRAME, handleEnterFrame);
			_frameRateDisplay = new TextField();
			_frameRateDisplay.text = "frame";
//			addChild(_frameRateDisplay);
		}
		
// ***** Functions - Event handlers ******************************************

		public function handleSkinLoaded(evt:Event):void
		{
//			var backdrop:Bitmap = _skin.getBackdrop();
			
//			addChild(backdrop);
//			changeMode(2);
		}
		
		public function handleEnterFrame(evt:Event):void
		{
			var newTimer:int = getTimer();
			var temp:int = newTimer - _lastUpdate;
			_lastUpdate = newTimer;
			var delta:Number = Math.min(Math.max(Number(temp)/1000.0, 0.01),1);
			_frameRateDisplay.text = "Framerate: " + (1.0/delta);

			if (_mode == MODE_GAME)
				_game.update(delta);
			else if (_mode == MODE_CAMPAIGN)
				_campaign.update(delta);
			else if (_mode == MODE_TITLE)
				_title.update(delta);
		}
		
		
		public function changeMode(n:int):void
		{
			// remove old mode
			switch(_mode) {
				case MODE_NONE:
					break;
				case MODE_ABOUT:
					removeChild(_about);
					break;
				case MODE_GAME:
					removeChild(_game);
					break;
				case MODE_CAMPAIGN:
					removeChild(_campaign);
					_campaign.setCampaign(_campaign.DEFAULT_CAMPAIGN);
					break;
				case MODE_SCORE:
					removeChild(_score);
					break;
				case MODE_TITLE:
					removeChild(_title);
					break;
				case MODE_EDIT:
					removeChild(_edit);
					_editing = n == MODE_CAMPAIGN;
			}
			
			// change to the new mode
			switch(n) {
				case MODE_NONE:
					break;
				case MODE_ABOUT:
					addChildAt(_about, 0);
					break;
				case MODE_GAME:
					_game.startGame(_skin);
					addChildAt(_game, 0);
					break;
				case MODE_CAMPAIGN:
					var cskin:Skin = new Skin();
					cskin.loadSkin();
					_campaign.startGame(cskin);
					addChildAt(_campaign, 0);
					break;
				case MODE_SCORE:
					addChildAt(_score, 0);
					break;
				case MODE_TITLE:
					addChildAt(_title, 0);
					break;
				case MODE_EDIT:
					addChildAt(_edit, 0);
			}
			
			_mode = n;
			if ((_editing) && (n == MODE_TITLE))
				changeMode(MODE_EDIT);

		}
		
		public function gameOver(n:int):void
		{
			_score.setLastScore(n);
			changeMode(MODE_SCORE);
		}
	}
}

