/*
Thirteen Spikes Unlimited version 0.8.0
Copyright (c) 2008-2009 Blazing Games Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.tsu
{
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.net.SharedObject;
	import flash.text.TextField;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;

	public class EditMenu
	extends Sprite
	{
// ***** CONSTANTS **********************************************************

		public static const HEADER_TEXT:String = "Thirteen Spikes Construction Set!";
		protected static const EXIT_TEXT:String = "Return to Title";
		protected static const TEST_TEXT:String = "Play Custom Set";
		protected static const PREV_TEXT:String = "<- Prev";
		protected static const NEW_TEXT:String = "Add Another Level";
		protected static const NEXT_TEXT:String = "Next ->";

// ***** VARIABLES **********************************************************

		protected var _tsu:ThirteenSpikesUnlimited;
		protected var _exitButton:TextButton;
		protected var _testButton:TextButton;
		protected var _prevButton:TextButton;
		protected var _newButton:TextButton;
		protected var _nextButton:TextButton;
		protected var _editBox:LevelEditBox;
		protected var _levelMessage:TextField;

		protected var _levels:Array;
		protected var _levelBeingEdited:int;
		protected var _sharedObject:SharedObject;
		
// ***** CONSTRUCTOR ********************************************************

		public function EditMenu(tsu:ThirteenSpikesUnlimited)
		{
			_tsu = tsu;
			_levels = new Array();
			
			var format:TextFormat = new TextFormat(null, 32, 0xFFFF00, null, null, null, null, null, TextFormatAlign.CENTER);
			var thirteen:TextField = new TextField();
			thirteen.x = 0;
			thirteen.y = 0;
			thirteen.width = 640;
			thirteen.height = 35;
			thirteen.defaultTextFormat = format;
			thirteen.text = HEADER_TEXT;
			addChild(thirteen);

			_exitButton = new TextButton(EXIT_TEXT,300);
			_exitButton.x = 10;
			_exitButton.y = 60;
			addChild(_exitButton);
			_exitButton.addEventListener(MouseEvent.MOUSE_UP, handleExit);

			_testButton = new TextButton(TEST_TEXT,300);
			_testButton.x = 320;
			_testButton.y = 60;
			addChild(_testButton);
			_testButton.addEventListener(MouseEvent.MOUSE_UP, handleTest);

			_prevButton = new TextButton(PREV_TEXT,150);
			_prevButton.x = 10;
			_prevButton.y = 360;
			addChild(_prevButton);
			_prevButton.addEventListener(MouseEvent.MOUSE_UP, handlePrev);

			_newButton = new TextButton(NEW_TEXT,300);
			_newButton.x = 170;
			_newButton.y = 360;
			addChild(_newButton);
			_newButton.addEventListener(MouseEvent.MOUSE_UP, handleNew);

			_nextButton = new TextButton(NEXT_TEXT,150);
			_nextButton.x = 480;
			_nextButton.y = 360;
			addChild(_nextButton);
			_nextButton.addEventListener(MouseEvent.MOUSE_UP, handleNext);

			_editBox = new LevelEditBox(tsu);
			_editBox.y = 120;
			addChild(_editBox);
			
			_levelMessage = new TextField();
			_levelMessage.x = 0;
			_levelMessage.y = 440;			
			_levelMessage.width = 640;
			_levelMessage.height = 35;
			_levelMessage.defaultTextFormat = format;
			_levelMessage.text = HEADER_TEXT;
			addChild(_levelMessage);
			
			try {
				_sharedObject = SharedObject.getLocal("ThirteenSpikesUnlimited");
			} catch (error:Error) {
				_sharedObject = null;
			}
			if (_sharedObject == null)
				handleNew(null);
			else {
				if (_sharedObject.data.levels == undefined) {
					_sharedObject.data.levels = new Array();
					_levels = _sharedObject.data.levels;
					handleNew(null);
				} else {
					_levels = _sharedObject.data.levels;
					_levelBeingEdited = _levels.length - 1;
					_editBox.setHeights(_levels[_levelBeingEdited]);
					updateUI();
				}
			}
		}

// ***** Functions - Event Handling *****************************************

		private function handleExit(evt:Event):void
		{
			_editBox.copyHeights(_levels[_levelBeingEdited]);
			_tsu.changeMode(ThirteenSpikesUnlimited.MODE_TITLE);
		}

		private function handleTest(evt:Event):void
		{
			_editBox.copyHeights(_levels[_levelBeingEdited]);
			_tsu._campaign.setCampaign(_levels);
			_tsu.changeMode(ThirteenSpikesUnlimited.MODE_CAMPAIGN);
		}

		private function handlePrev(evt:Event):void
		{
			_editBox.copyHeights(_levels[_levelBeingEdited]);
			--_levelBeingEdited;
			_editBox.setHeights(_levels[_levelBeingEdited]);
			updateUI();
		}

		private function handleNew(evt:Event):void
		{
			var a:Array = new Array();
			for (var cntr:int = 0; cntr < LevelEditBox.NUM_SPIKES; ++cntr)
				a[cntr] = LevelEditBox.NUM_STEPS;
			if (evt != null)
				_editBox.copyHeights(_levels[_levelBeingEdited]);
			_levels.push(a);
			_levelBeingEdited = _levels.length - 1;
			_editBox.setHeights(_levels[_levelBeingEdited]);
			updateUI();
		}

		private function handleNext(evt:Event):void
		{
			_editBox.copyHeights(_levels[_levelBeingEdited]);
			++_levelBeingEdited;
			_editBox.setHeights(_levels[_levelBeingEdited]);
			updateUI();
		}

// ***** Functions - UI Management ******************************************

		public function updateUI():void
		{
			_prevButton.visible = _levelBeingEdited > 0;
			_nextButton.visible = _levelBeingEdited < (_levels.length-1);
			_levelMessage.text = "Editing level " + (_levelBeingEdited+1) + " of " + _levels.length;
		}
	}
}
