/*
Thirteen Spikes Unlimited version 0.4.0
Copyright (c) 2008-2009 Blazing Games Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.tsu
{
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.geom.Point;
	import flash.geom.Rectangle;

	public class ImageStrip
	extends Bitmap
	{
// ***** CONSTANTS **********************************************************


// ***** VARIABLES **********************************************************

		protected var _sourceData:BitmapData;
		protected var _frameWidth:int;
		protected var _frameHeight:int;
		protected var _animated:Boolean;
		protected var _delayPerFrame:Number;
		protected var _totalFrames:int;
		protected var _currentFrame:int;
		protected var _loopAnimation:Boolean;
		
		protected var _startFrame:int;
		protected var _endFrame:int;
		protected var _currentDelay:Number;

// ***** CONSTRUCTOR ********************************************************

		public function ImageStrip(sourceData:BitmapData, 
				frameWidth:int, 
				frameHeight:int,
				animate:Boolean = false,
				totalFrames:int = 1,
				delayPerFrame:Number = 0.1	)
		{
			_sourceData = sourceData;
			_frameWidth = frameWidth;
			_frameHeight = frameHeight;
			_animated = animate;
			_delayPerFrame = delayPerFrame;
			_totalFrames = totalFrames;
			_currentFrame = 0;
			_startFrame = 0;
			_endFrame = totalFrames - 1;
			bitmapData = new BitmapData(frameWidth, frameHeight);
			_currentDelay = 0;
			_loopAnimation = true;
			draw();
		}

// ***** Functions - Rendering **********************************************

		public function draw():void
		{
			var sourceRect:Rectangle = new Rectangle(_frameWidth * _currentFrame, 0, _frameWidth, _frameHeight);
			var destPoint:Point = new Point(0,0);
			bitmapData.copyPixels(_sourceData, sourceRect, destPoint);
		}
		
// ***** Functions - Getting and Setting ************************************

		public function setFrame(newFrame:int):void
		{
			if (_currentFrame != newFrame) {
				_currentFrame = newFrame;
				draw();
			}
		}
		
		public function setAnimation(startFrame:int, endFrame:int, loop:Boolean = true):void
		{
			_startFrame = startFrame;
			_endFrame = endFrame;
			_loopAnimation = loop;
			_currentFrame = startFrame;
			_currentDelay = 0;
			_animated = (startFrame != endFrame);
			draw();
		}

		/** returns true if animation still running, false if done */
		public function updateAnimation(delta:Number):Boolean
		{
			if (_animated == false)
				return false;
			_currentDelay += delta;
			if (_currentDelay > _delayPerFrame) {
				_currentDelay = 0;
				++_currentFrame;
				if (_currentFrame > _endFrame)
					if (_loopAnimation == false) {
						_currentFrame = _endFrame;
						return false;
					} else
						_currentFrame = _startFrame;
				draw();
			}
			return true;
		}
	}
}
