/*
Thirteen Spikes Unlimited version 0.4.0
Copyright (c) 2008-2009 Blazing Games Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.tsu
{
	import flash.display.Bitmap;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;

	import com.blazinggames.tsu.skins.Skin;

	public class Level
	extends Sprite
	{
// ***** CONSTANTS **********************************************************

		public static const LEVEL_MIDPOINT:int = 240;
		public static const LEVEL_LEFTEDGE:int = -5;
		public static const NUM_SPIKES:int = 13;

		public static const STEP_SIZE:int = 10;
		public static const NUM_STEPS:int = LEVEL_MIDPOINT / STEP_SIZE;

		public static const PIXELS_PER_SECOND:Number = 20;
		public static const MOVEMENT_START_DELAY:Number = 2;

// ***** VARIABLES **********************************************************

		protected var _tsu:ThirteenSpikesUnlimited;
		protected var _spikes:Array;
		protected var _platforms:Array;
		protected var _playTime:Number;
		protected var _endTime:Number;
		protected var _playInProgress:Boolean;
		protected var _player:Player;
		protected var _safe:int;
		protected var _bspot:int;
		protected var _curLevel:int;
		protected var _levelMessage:TextField;

		protected var _lastSkin:Skin;
		protected var _backdrop:Bitmap;

// ***** CONSTRUCTOR ********************************************************

		public function Level(tsu:ThirteenSpikesUnlimited)
		{
			_tsu = tsu;
			_spikes = new Array(NUM_SPIKES);
			_platforms = new Array(NUM_SPIKES);
			_player = new Player();
			_backdrop = null;
			
			var col:int = LEVEL_LEFTEDGE;
			for (var cntr:int = 0; cntr < NUM_SPIKES; ++cntr) {
				_spikes[cntr] = new Spike();
				_spikes[cntr].x = col;
				_spikes[cntr].setStartingSpot(-LEVEL_MIDPOINT*3);
				addChild(_spikes[cntr]);
				_platforms[cntr] = new Platform();
				_platforms[cntr].x = col;
				_platforms[cntr].y = LEVEL_MIDPOINT;
				addChild(_platforms[cntr]);
				col += Skin.SPIKE_WIDTH;
			}
			addChild(_player);
			_curLevel = 1;
			var format:TextFormat = new TextFormat(null, 64, 0xFFFFFF, null, null, null, null, null, TextFormatAlign.CENTER);
			_levelMessage = new TextField();
			_levelMessage.x = 0;
			_levelMessage.y = LEVEL_MIDPOINT - 32;
			_levelMessage.width = 640;
			_levelMessage.height = 64;
			_levelMessage.defaultTextFormat = format;
			_levelMessage.text = "Level 1";
			addChild(_levelMessage);
		}

// ***** Functions - Event Handling *****************************************

		public function handleMouseUp(evt:MouseEvent):void
		{
			var col:int = (evt.stageX - LEVEL_LEFTEDGE) / Skin.SPIKE_WIDTH;
			if ( (col >= 0) && (col < NUM_SPIKES) ) {
				_bspot = col;
				_player.x = _platforms[col].x + (Skin.PLATFORM_WIDTH - Skin.PLAYER_WIDTH)/2;
				_player.y = _platforms[col].y - Skin.PLAYER_HEIGHT;
				_player.startTeleport();
			}
		}

// ***** Functions - Rendering **********************************************

		public function update(delta:Number):void
		{
			_playTime += delta;
			_levelMessage.visible = _playTime < MOVEMENT_START_DELAY;
			var spikeOffset:Number = Math.max(0, Math.min(LEVEL_MIDPOINT, 
					(_playTime - MOVEMENT_START_DELAY) * PIXELS_PER_SECOND));
			for (var cntr:int = 0; cntr < NUM_SPIKES; ++cntr) {
				_spikes[cntr].y = _spikes[cntr].getStartingSpot() + spikeOffset;
				_spikes[cntr].updateAnimation(delta);
				_platforms[cntr].updateAnimation(delta);
			}
			_player.updateAnimation(delta);

			if ((_playInProgress) && (spikeOffset > (LEVEL_MIDPOINT-Skin.PLAYER_HEIGHT))) {
				_playInProgress = false;
				removeEventListener(MouseEvent.MOUSE_UP, handleMouseUp);
				if (_bspot != _safe)
					_player.startDeath();
			}
			if (_playTime > _endTime) {
				if (_bspot == _safe) {
					++_curLevel;
					_levelMessage.text = "Level " + _curLevel;
					generateLevel(Math.min(_curLevel, 24));
				} else
					_tsu.gameOver(_curLevel);
			}
		}

// ***** Functions - Level manipulation *************************************

		public function startGame(skin:Skin):void
		{
			var theSkin:Skin = skin;
			if (skin == null)
				return;
			if (_lastSkin != theSkin) {
				_lastSkin = theSkin;
				if (_backdrop != null)
					removeChild(_backdrop);
				_backdrop = theSkin.getBackdrop();
				addChildAt(_backdrop, 0);
				for (var cntr:int = 0; cntr < NUM_SPIKES; ++cntr) {
					_spikes[cntr].setImageStrip(theSkin.createSpikeStrip());
					_platforms[cntr].setImageStrip(theSkin.createPlatformStrip());
				}
				_player.setImageStrip(theSkin.createPlayerStrip(), 
						theSkin.getFrameIndex(Skin.FRAMEINDEX_BALLOON_APPEAR_START),
						theSkin.getFrameIndex(Skin.FRAMEINDEX_BALLOON_APPEAR_END),
						theSkin.getFrameIndex(Skin.FRAMEINDEX_BALLOON_STATIONARY_START),
						theSkin.getFrameIndex(Skin.FRAMEINDEX_BALLOON_STATIONARY_END),
						theSkin.getFrameIndex(Skin.FRAMEINDEX_BALLOON_POP_START),
						theSkin.getFrameIndex(Skin.FRAMEINDEX_BALLOON_POP_END));
			}
			_curLevel = 1;
			_levelMessage.text = "Level 1";
			generateLevel(1);
		}
		
		public function generateLevel(deviation:int):void
		{
			var step:int = Math.random() * NUM_STEPS;
			var minStep:int, maxStep:int, stepCount:int;
			
			// randomly build the level
			for (var cntr:int = 0; cntr < NUM_SPIKES; ++cntr) {
				minStep = Math.max(step - deviation, 1);
				maxStep = Math.min(step + deviation, NUM_STEPS);
				stepCount = maxStep - minStep + 1;
				step = Math.random() * stepCount + minStep;
				_platforms[cntr].y = LEVEL_MIDPOINT + STEP_SIZE * step;
				_spikes[cntr].setStartingSpot(_platforms[cntr].y - LEVEL_MIDPOINT * 3);
			}
			
			// set the safe spot and balloon spot
			_safe = Math.random() * NUM_SPIKES;
			_spikes[_safe].setStartingSpot(_spikes[_safe].getStartingSpot() - Skin.PLAYER_HEIGHT);
			_bspot = Math.random() * NUM_SPIKES;
			_player.x = _platforms[_bspot].x + (Skin.PLATFORM_WIDTH - Skin.PLAYER_WIDTH)/2;
			_player.y = _platforms[_bspot].y - Skin.PLAYER_HEIGHT;
			_player.startTeleport();
			_playTime = 0;
			_endTime = Number(LEVEL_MIDPOINT) / Number(PIXELS_PER_SECOND) + 2 * MOVEMENT_START_DELAY;
			_playInProgress = true;
			addEventListener(MouseEvent.MOUSE_UP, handleMouseUp);
		}
	}
}
