/*
Thirteen Spikes Unlimited version 0.8.0
Copyright (c) 2008-2009 Blazing Games Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.tsu
{
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;

	import com.blazinggames.tsu.skins.Skin;

	public class LevelEditBox
	extends Sprite
	{
// ***** CONSTANTS **********************************************************

		public static const STEP_SIZE:int = 10;
		public static const NUM_STEPS:int = 24;
		public static const HEIGHT:int = NUM_STEPS * STEP_SIZE;
		public static const NUM_SPIKES:int = 13;

		public static const BORDER_COLOR:int = 0xFFFF80;
		public static const BACKGROUND_COLOR:int = 0x500050;
		public static const PLATFORM_FILL_COLOR:int = 0x0099CC;
		
// ***** VARIABLES **********************************************************

		protected var _tsu:ThirteenSpikesUnlimited;
		protected var _platforms:Array;

// ***** CONSTRUCTOR ********************************************************

		public function LevelEditBox(tsu:ThirteenSpikesUnlimited)
		{
			_tsu = tsu;
			_platforms = new Array(NUM_SPIKES);
			
			var col:int = 0;
			for (var cntr:int = 0; cntr < NUM_SPIKES; ++cntr) {
				_platforms[cntr] = NUM_STEPS;
			}
			addEventListener(MouseEvent.MOUSE_UP, handleMouseUp);
			draw();
		}

// ***** Functions - Event Handling *****************************************

		public function handleMouseUp(evt:MouseEvent):void
		{
			var col:int = (evt.localX) / Skin.SPIKE_WIDTH;
			if ( (col >= 0) && (col < NUM_SPIKES) ) {
				_platforms[col] = Math.floor(evt.localY/STEP_SIZE);
			}
			draw();
		}

// ***** Functions - Rendering **********************************************

		public function draw():void
		{
			graphics.clear();
			graphics.beginFill(BACKGROUND_COLOR);
            graphics.lineStyle(3, BORDER_COLOR); 
			graphics.drawRect(0, 0, 640, HEIGHT);
            graphics.endFill();

			var col:int = 0;
			var h:int;
			for (var cntr:int = 0; cntr < NUM_SPIKES; ++cntr) {
				graphics.beginFill(PLATFORM_FILL_COLOR);
				h = (NUM_STEPS - _platforms[cntr]) * STEP_SIZE;
				graphics.drawRect(col, HEIGHT-h,Skin.SPIKE_WIDTH, h);
				col += Skin.SPIKE_WIDTH;
				graphics.endFill();
			}
		}

// ***** Functions - Level management ***************************************

		public function copyHeights(a:Array):void
		{
			for (var cntr:int = 0; cntr < NUM_SPIKES; ++cntr) {
				a[cntr] = _platforms[cntr];
			}
		}
		
		public function setHeights(a:Array):void
		{
			for (var cntr:int = 0; cntr < NUM_SPIKES; ++cntr) {
				_platforms[cntr] = a[cntr];
			}
			draw();
		}
	}
}

